#ifndef MidiMon_hxx
#define MidiMon_hxx

#include <CLAM/InControl.hxx>
#include <CLAM/OutControl.hxx>
#include <CLAM/Processing.hxx>
#include <portmidi.h>
#include <porttime.h>

/* Defines needed by PortMIDI */ 
#define INPUT_BUFFER_SIZE 100
#define OUTPUT_BUFFER_SIZE 0
#define DRIVER_INFO NULL
#define TIME_PROC ((long (*)(void *)) Pt_Time)
#define TIME_INFO NULL
#define TIME_START Pt_Start(1, 0, 0) /* timer started w/millisecond accuracy */


/* MidiMon Configuration Object */
class MidiMonConfig : public CLAM::ProcessingConfig
{
public:
	DYNAMIC_TYPE_USING_INTERFACE( MidiMonConfig, 1, ProcessingConfig );
	DYN_ATTRIBUTE( 0, public, int, MidiDevice );

protected:
	void DefaultInit(){
		AddAll();
		UpdateData();
	}
};
/*  End MidiMon Configuration Object */

class MidiMon : public CLAM::Processing
{ 
	CLAM::OutControl midiMessage;
	CLAM::OutControl midiData1;
	CLAM::OutControl midiData2;
		
public:
	
	typedef MidiMonConfig Config;

	const char* GetClassName() const { return "MidiMon"; }
	MidiMon(const Config& config = Config()) 
		: midiMessage("MIDI Message", this),
		  midiData1("MIDI Data 1", this),
		  midiData2("MIDI Data 2", this)
	{
		Configure( config );
	}
 
	const CLAM::ProcessingConfig & GetConfig() const
	{
		return mConfig;
	}
		
	bool Do()
	{
        length = Pm_Read(midi,buffer, 1);
	        if (length > 0) {
				midiMessage.SendControl((float)Pm_MessageStatus(buffer[0].message));
				midiData1.SendControl((float)Pm_MessageData1(buffer[0].message));
				midiData2.SendControl((float)Pm_MessageData2(buffer[0].message));
	        } else {
	         // std::cout << "Ouch! " << length << std::endl;
	        }

		// Tell the ports this is done
		return true;
	}

	
	protected:
		bool ConcreteConfigure(const CLAM::ProcessingConfig & config)
		{
			/* Work arround for "Bad pointer" error */
			Pm_Close(midi);
			
			/* list device information */
		    for (int i = 0; i < Pm_CountDevices(); i++) {
		        const PmDeviceInfo *info = Pm_GetDeviceInfo(i);
		            printf("%d: %s, %s", i, info->interf, info->name);
		            if (info->input) printf(" (input)");
		            if (info->output) printf(" (output)");
		            printf("\n");
		    }
		
			CopyAsConcreteConfig(mConfig, config);
			MidiDevice = mConfig.GetMidiDevice();
			TIME_START;
			
			status = Pm_OpenInput(&midi, 
		                 0,
		                 DRIVER_INFO, 
		                 INPUT_BUFFER_SIZE, 
		                 TIME_PROC, 
		                 TIME_INFO);
			Pm_SetFilter(midi, PM_FILT_ACTIVE | PM_FILT_CLOCK);
			
			/* empty the buffer after setting filter, just in case anything
		       got through */
		    /*while (Pm_Poll(midi)) {
		        Pm_Read(midi, buffer, 1);
		    }*/
			return true; // Configuration ok
		}
		
	private:
		int MidiDevice;
		PmStream *midi;
		PmError status, length;
		PmEvent buffer[1];
		Config mConfig;

};
#endif // MidiMon_hxx